package com.lambdazen.bitsy.index;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public abstract class BitsyIndex<T> {
    Map<Object, LinkedList<T>> index;
    
    public BitsyIndex() {
        this.index = new ConcurrentHashMap<Object, LinkedList<T>>();
    }
    
    public abstract Object getValue(T bean);

    public void load(Iterator<T> initialContents) {
        while (initialContents.hasNext()) {
            T elem = initialContents.next();
            add(elem);
        }
    }
    
    public LinkedList<T> get(Object value) {
        return index.get(value);
    }
    
    public void add(T bean) {
        Object value = getValue(bean);
        if (value == null) {
            // Nothing to do
            return;
        }
        
        LinkedList<T> list = index.get(value);
        if (list == null) {
            list = new LinkedList<T>();
            index.put(value, list);
        }

        // No need to synchronize, because there is a read-write lock
        list.add(bean);
    }
    
    public void remove(T bean) {
        Object value = getValue(bean);
        if (value == null) {
            // Nothing to do
            return;
        }
        
        LinkedList<T> list = index.get(value);
        if (list != null) {
            // No need to synchronize, because there is a read-write lock
            list.remove(bean);
            
            if (list.isEmpty()) {
                index.remove(value);
            }
        }
    }
}
