package com.lambdazen.bitsy.store;

import java.io.Serializable;
import java.util.Map;
import java.util.UUID;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.lambdazen.bitsy.BitsyState;
import com.lambdazen.bitsy.IEdge;
import com.nesscomputing.uuid.NessUUID;

@JsonPropertyOrder({"class", "id", "v", "s", "o", "l", "i", "p"})
public class EdgeBean implements IEdge, Serializable {
    
    private static final long serialVersionUID = -5962479601393604124L;
    
    private UUID id;
    private Map<String, Object> properties;
    private String label;
    private UUID outVertexId;
    private UUID inVertexId;
    private int version;
    private BitsyState state;
    
    public EdgeBean(UUID id, Map<String, Object> properties, int version, String label, UUID outVertexId, UUID inVertexId, BitsyState state) {
        this.id = id;
        this.properties = properties;
        this.version = version;
        this.label = label;
        this.outVertexId = outVertexId;
        this.inVertexId = inVertexId;
        this.state = state;
    }
    
    @JsonCreator
    public EdgeBean(@JsonProperty("id") String idStr, 
            @JsonProperty("p") Map<String, Object> properties, 
            @JsonProperty("v") int version, 
            @JsonProperty("l") String label, 
            @JsonProperty("o") String outVertexIdStr, 
            @JsonProperty("i") String inVertexIdStr, 
            @JsonProperty("s") BitsyState state) {
        this.id = NessUUID.fromString(idStr);
        this.properties = properties;
        this.version = version;
        this.label = label;
        this.outVertexId = NessUUID.fromString(outVertexIdStr);
        this.inVertexId = NessUUID.fromString(inVertexIdStr);
        this.state = state;
    }
    
    public String toString() {
        return "EdgeBean(id = " + id + ", props " + getProperties() + ", version = " + version + ", label = " + label + ", outV = " + outVertexId + ", inV = " + inVertexId + ")";
    }

    @JsonIgnore
    public UUID getId() {
        return id;
    }
    
    @JsonProperty("id")
    public String getIdStr() {
        return id.toString();
    }

    @JsonProperty("p")
    public Map<String, Object> getProperties() {
        return properties;
    }

    @JsonProperty("v")
    public int getVersion() {
        return version;
    }

    @JsonIgnore
    public UUID getInVertexId() {
        return inVertexId;
    }
    
    @JsonProperty("i")
    public String getInVertexIdStr() {
        return NessUUID.toString(inVertexId);
    }

    @JsonIgnore
    public UUID getOutVertexId() {
        return outVertexId;
    }
    
    @JsonProperty("o")
    public String getOutVertexIdStr() {
        return NessUUID.toString(outVertexId);
    }

    @JsonProperty("l")
    public String getLabel() {
        return label;
    }
    
    @JsonProperty("s")
    public BitsyState getState() {
        return state;
    }
}
